/*
 * JSatBrowser.java
 * =====================================================================
 * Copyright (C) 2008 Shawn E. Gano
 * 
 * This file is part of JSatTrak.
 * 
 * JSatTrak is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JSatTrak is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JSatTrak.  If not, see <http://www.gnu.org/licenses/>.
 * =====================================================================
 *
 * Created on July 23, 2007, 4:51 PM
 */

package jsattrak.gui;

import java.io.File;
import java.util.Hashtable;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;
import jsattrak.utilities.SatBrowserTleDataLoader;
import jsattrak.utilities.TLE;
import jsattrak.utilities.TLEDownloader;
import jsattrak.utilities.TreeTransferHandler;

/**
 *
 * @author  ganos
 */
public class JSatBrowser extends javax.swing.JDialog implements java.io.Serializable
{
    
    DefaultTreeModel treeModel;
    
    DefaultMutableTreeNode topTreeNode; // top node
    
    // hashtable with tle's
    private Hashtable<String,TLE> tleHash;
    
    java.awt.Frame parent;
    
    /** Creates new form JSatBrowser */
    public JSatBrowser(java.awt.Frame parent, boolean modal)
    {
        super(parent, modal);
        initComponents();
        this.parent = parent;
        
        // setup the tree --
        // JTree tree = new JTree(System.getProperties());
        //satTree.setModel( System.getProperties() );
        
        // -- Create Tree List Based on TLEDownloader values
                
        topTreeNode = new DefaultMutableTreeNode("Satellites");
        //createNodes(top);
        //tree = new JTree(top);
        
        // create a hashmap of TLEs with key as text from tree
        tleHash = new Hashtable<String,TLE>();
        
        treeModel = new DefaultTreeModel(topTreeNode); // create tree model using root node
        satTree.setModel(treeModel); // set the tree's model
        
        SatBrowserTleDataLoader sbtdl = new SatBrowserTleDataLoader(parent, topTreeNode, tleHash, tleOutputTextArea, satTree);
        
        sbtdl.execute();
        

        // Drag and Drop Handler
        // setup transfer handler
        satTree.setTransferHandler(new TreeTransferHandler(tleHash));
        
        // allow mutiple selections
        satTree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        
        // if local files exist wait for thread to finish before exiting
        TLEDownloader tleDownloader = new TLEDownloader();
        if( (new File(tleDownloader.getLocalPath()).exists()) && (new File(tleDownloader.getTleFilePath(0)).exists()) )
        {
            int i=0;
            while(!sbtdl.isDone())
            {
                
                try
                {
                    Thread.sleep(50); // sleep for a little bit to wait for the thread
                    //i++;
                }
                catch(Exception e)
                {
                    //e.printStackTrace();
                }
            }
            //System.out.println("trys:"+ i);
        } // wait for thread if needed
        
    
    } // constructor
    

        
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        tleOutputTextArea = new javax.swing.JTextArea();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        satTree = new javax.swing.JTree();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Satellite Browser");
        jLabel1.setFont(new java.awt.Font("Arial", 1, 18));
        jLabel1.setText("Satellite Browser");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jLabel1)
                .addContainerGap(106, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jLabel1)
        );

        tleOutputTextArea.setColumns(20);
        tleOutputTextArea.setRows(5);
        tleOutputTextArea.setToolTipText("Satellite TLE");
        jScrollPane3.setViewportView(tleOutputTextArea);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 62, Short.MAX_VALUE)
        );

        satTree.setToolTipText("Drag Satellite(s) over to Satellite List");
        satTree.setDragEnabled(true);
        satTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener()
        {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt)
            {
                satTreeValueChanged(evt);
            }
        });

        jScrollPane1.setViewportView(satTree);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 280, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void satTreeValueChanged(javax.swing.event.TreeSelectionEvent evt)//GEN-FIRST:event_satTreeValueChanged
    {//GEN-HEADEREND:event_satTreeValueChanged
        // some other sat was selected
        
        if(satTree.getSelectionCount() > 0)
        {
            if( tleHash.containsKey(satTree.getLastSelectedPathComponent().toString()) )
            {
                TLE selectedTLE = tleHash.get(satTree.getLastSelectedPathComponent().toString());
                
                tleOutputTextArea.setText( selectedTLE.getLine1() + "\n" + selectedTLE.getLine2() );
            }
            else // clear text area
            {
                tleOutputTextArea.setText("");
            }
        } // something is selected

    }//GEN-LAST:event_satTreeValueChanged
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                new JSatBrowser(new javax.swing.JFrame(), true).setVisible(true);
            }
        });
    }

    public Hashtable<String, TLE> getTleHash()
    {
        return tleHash;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTree satTree;
    private javax.swing.JTextArea tleOutputTextArea;
    // End of variables declaration//GEN-END:variables
    
}
